package examples.hospital;

import helperclasses.*;
import ttclasses.*;
import java.util.*;

/*****
 * The <tt>ClashConstraint</tt> class manages constraints of the type
 * "no clashes allowed", i.e. it reports a constraint violation if there
 * are one or more clashes among the events of the resource specified
 * by <tt>owner</tt>.
 *****/
public class ClashConstraint extends HardConstraint {
  private Random random;

  public ClashConstraint() {
    name = "ClashConstraint";
    random = GlobalObjects.random;
  }

  public ConstraintViolation compute(PlanningClass owner) {
    try {
      int penalty = 0;
      ArrayList info = new ArrayList();
      List assignments = ((Resource)owner).getAssignments();
      assignments = GlobalMethods.mixList(assignments);
      int size = assignments.size();
      for (int j = 0; j < size - 1; j++) {
        Assignment ass1 = (Assignment)assignments.get(j);
        for (int k = j + 1; k < size; k++) {
          Assignment ass2 = (Assignment)assignments.get(k);
          if (ass1.event.clashes(ass2.event)) {
            penalty += 200;
            info.add(ass2.resList);
            ass1.event.penaltyPoints += 100;
            ass2.event.penaltyPoints += 100;
          }
        }
      }
      if (penalty > 0)
        return new ConstraintViolation(this, owner, "Clash at " + owner, penalty, info);
      else
        return null;
    }
    catch (Exception e) {
      System.out.println(e.toString());
      return null;
    }
  } // end of compute(PlanningClass)

  public boolean repair(ConstraintViolation violation) {
    ArrayList info = (ArrayList)violation.information;
    for (int i = info.size() - 1; i >= 0; i--) {
      ResourceList rl = (ResourceList)info.get(i);
      rl.freeResource((Resource)violation.owner);
    }
    return true;
  }

} // end of class ClashConstraint

