package examples.maedchen;

import ttclasses.*;
import java.util.*;

/*****
 * This is the canonical constraint that assures that we do not have too many or too few
 * resources per event (project).
 *****/
public class MinMaxConstraint extends HardConstraint {

  public MinMaxConstraint() {
    name = "MinMaxConstraint";
  }

  public ConstraintViolation compute(PlanningClass owner) {
    int size;
    int penalty = 0;
    Event ev = (Event)owner;
    List l = new ArrayList();
    for (int i = 0; i < ev.resources.size(); i++) {
      ResourceList rl = (ResourceList)ev.resources.get(i);
      size = rl.assignedCount();
      if (size > rl.getMax()) {
        penalty += (size - rl.getMax()) * 10;
        l.add(rl);
      }
      if (size < rl.getMin()) {
        penalty += (rl.getMin() - size) * 10;
        l.add(rl);
      }
    }
    ev.penaltyPoints += penalty;
    if (penalty > 0)
      return new ConstraintViolation(this, owner, "MinMaxConstraint Violation at " + ev.name, penalty, l);
    else
      return null;
  } // end of compute(PlanningClass)

  public boolean repair(ConstraintViolation violation) {
    ResourceList rl;
    List list = (List)violation.information;
    int k = list.size();
    for (int i = 0; i < k; i++) {
      rl = (ResourceList)list.get(i);
      if (rl.assignedCount() > rl.getMax())
        rl.freeRandom();
      else if (rl.assignedCount() < rl.getMin())
        rl.assignRandom();
    }
    return true;
  }

} // end of class MinMaxConstraint
