package examples.school;

import helperclasses.*;
import ttclasses.*;

import java.util.*;

/*****
 * This is the usual Clash Constraint, applied to the Highschool Timetabling problem.
 * It features a repair method.
 *****/

class ClashConstraint extends HardConstraint {
  private Random random;

  public ClashConstraint() {
    name = "ClashConstraint";
    random = GlobalObjects.random;
  }

  public ConstraintViolation compute(PlanningClass owner) {
    int j, k;
    Resource res = (Resource)owner;
    ArrayList info = new ArrayList();
    ArrayList l = (ArrayList)GlobalMethods.mixList(res.getEvents());
    Event ev1, ev2;
    int size = l.size();
    double penalty = 0;
    for (j = 0; j < size - 1; j++) {
      ev1 = (Event)l.get(j);
      for (k = j + 1; k < size; k++) {
        ev2 = (Event)l.get(k);
        if (ev1.clashes(ev2)) {
          info.add(ev2);
          penalty += 200;
          ev1.penaltyPoints += 100;
          ev2.penaltyPoints += 100;
        }
      }
    }
    if (penalty > 0)
      return new ConstraintViolation(this, owner, "ClashViolation at " + res.name, penalty, info);
    return null;
  } // end of compute(PlanningClass)

  public boolean repair(ConstraintViolation violation) {
    int i;
    ArrayList list = (ArrayList)violation.information;
    for (i = list.size() - 1; i >= 0; i--) {
      Event e = (Event)list.get(i);
      if ((violation.owner instanceof Room) && (random.nextBoolean())) {
        ResourceList rl = e.getResourceList("Room");
        rl.mutate();
      }
      else {
        ResourceList rl = e.getResourceList("TimeSlots");
        rl.mutate(); rl.mutate();
      }
    }
    return true;
  } // end of repair(ConstraintViolation)

} // end of class ClashConstraint



class RoomClashConstraint extends HardConstraint {
  private Random random;

  public RoomClashConstraint() {
    name = "RoomClashConstraint";
    random = GlobalObjects.random;
  }

  public ConstraintViolation compute(PlanningClass owner) {
    int j, k;
    Resource res = (Resource)owner;
    ArrayList info = new ArrayList();
    ArrayList l = (ArrayList)GlobalMethods.mixList(res.getEvents());
    Event ev1, ev2;
    int size = l.size();
    double penalty = 0;
    for (j = 0; j < size - 1; j++) {
      ev1 = (Event)l.get(j);
      for (k = j + 1; k < size; k++) {
        ev2 = (Event)l.get(k);
        if (ev1.clashes(ev2)) {
          info.add(ev2);
          penalty += 100;
          ev1.penaltyPoints += 50;
          ev2.penaltyPoints += 50;
        }
      }
    }
    if (penalty > 0)
      return new ConstraintViolation(this, owner, "ClashViolation at " + res.name, penalty, info);
    return null;
  } // end of compute(PlanningClass)

  public boolean repair(ConstraintViolation violation) {
    int i;
    ArrayList list = (ArrayList)violation.information;
    for (i = list.size() - 1; i >= 0; i--) {
      Event e = (Event)list.get(i);
      if ((violation.owner instanceof Room) && (random.nextBoolean())) {
        ResourceList rl = e.getResourceList("Room");
        rl.mutate();
      }
      else {
        ResourceList rl = e.getResourceList("TimeSlots");
        rl.mutate(); rl.mutate();
      }
    }
    return true;
  } // end of repair(ConstraintViolation)

} // end of class RoomClashConstraint

