package tools.frontend;

import ga.*;
import examples.*;
import examples.hospital.*;
import examples.maedchen.*;
import examples.school.*;

import java.util.*;

/*****
 * This thread manages the evolution process. Creating a thread is
 * necessary because evolution runs in parallel with the ordinary
 * window/frame stuff.
 */
public final class EvolutionThread extends Thread {
  protected GeneticAlgorithm ga;
  protected GeneticIndividual adam;
  protected MainFrame parent;

  boolean terminated, paused;

  /** The constructor takes two strings and initializes the GA from them. **/
  public EvolutionThread(MainFrame parent, String species, String init) {
    this.parent = parent;
    Map parameters = null;
    adam = null;
    if (species.equals("Mdchen und Technik")) {
      adam = new MaedchenUndTechnik(init + "/Kurse.txt", init + "/Kursplan.txt",
                                    init + "/Teilnehmer.txt");
    }
    else if (species.equals("Schulstundenplan")) {
      adam = new School(init);
    }
    else if (species.equals("Krankenhaus")) {
      adam = new Hospital(init);
    }
    if (adam != null) {
      ga = new GeneticAlgorithm(adam, parameters);
    }

    terminated = paused = false;
  } // end of EvolutionThread(String, String)

  public void run() {
    long startTime = System.currentTimeMillis();
    long lastUpdate = 0;
    while (!terminated) {
      ga.evolve();
      if (terminated) break;
      long now = System.currentTimeMillis();
      if (lastUpdate == 0) {
        lastUpdate = now;
        GeneticIndividual best = ga.getBestSoFar();
        parent.addData(ga.getGenerationCount(), ga.getBestFitness(), ga.getAverageFitness());
      }
      else if (now > lastUpdate + 5000) {  // update information every 10 seconds
        lastUpdate += 5000;
        GeneticIndividual best = ga.getBestSoFar();
        parent.addData(ga.getGenerationCount(), ga.getBestFitness(), ga.getAverageFitness());
      }
      while (paused)  // do nothing while in pause mode
        try { sleep(1); } catch (InterruptedException ie) { }
    }
  } // end of run()

} // end of class EvolutionThread


