/**
 * Implementation of class "MinMaxConstraint".
 *
 * Author: Stefan Buettcher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 **/

package ttclasses;

import java.util.*;

/*****
 * This is the canonical constraint that assures that we do not have too many or too few
 * resources per event. This is done for each ResourceList sequentially.
 *****/
public class MinMaxConstraint extends Constraint {

  public MinMaxConstraint() {
  }

  public ConstraintViolation compute(PlanningClass owner) {
    int size;
    int akku = 0;
    Event ev = (Event)owner;
    List l = new ArrayList();
    for (int i = 0; i < ev.resources.size(); i++) {
      ResourceList rl = (ResourceList)ev.resources.get(i);
      size = rl.assignedCount();
      if (size > rl.getMax()) {
        akku += (size - rl.getMax());
        l.add(rl);
      }
      if (size < rl.getMin()) {
        akku += (rl.getMin() - size);
        l.add(rl);
      }
    }
    if (akku == 0)
      return null;
    else
      return new ConstraintViolation(this, owner, "MinMaxViolation at " + ev.name, akku, l);
  } // end of compute(PlanningClass)

  public boolean repair(ConstraintViolation violation) {
    List l = (List)violation.information;
    for (int i = 0; i < l.size(); i++)
      ((ResourceList)l.get(i)).mutate();
    return false;
  }

  public int getType() {
    return HARD_CONSTRAINT;
  }

} // end of class MinMaxConstraint
