/**
 * Implementation of class "ResourceType".
 *
 * Author: Stefan Buettcher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 **/

package ttclasses;

import java.util.*;
import helperclasses.*;

/*****
 * This class encapsulates all resource components of a certain type,
 * e.g. all time slots, all rooms, all teachers, ...
 *****/
public class ResourceType {
  /** Name of this resource type, e.g. "TimeSlots". **/
  protected String name;
  /** List of all resources that are of a certain type. **/
  protected List components;

  public ResourceType(String aName) {
    name = aName;
    components = new SortedList();
  }

  /** Returns the list of all resources of this resource type. **/
  public final List getComponents() {
    return components;
  }

  /** Adds one resource to the list. **/
  public final void add(Resource res) {
    components.add(res);
  }
  
  /** Gets the component with the given name. **/
  public final Resource get(String name) {
    return (Resource)((SortedList)components).get(name);
  }

} // end of class ResourceType


